# chrono - agendas system
# Copyright (C) 2016  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json

from django.core.urlresolvers import reverse, reverse_lazy
from django.http import HttpResponse
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_text
from django.views.generic import (DetailView, CreateView, UpdateView,
        ListView, DeleteView)

from chrono.agendas.models import Agenda, Event

from .forms import EventForm


class HomepageView(ListView):
    template_name = 'chrono/manager_home.html'
    model = Agenda

homepage = HomepageView.as_view()


class AgendaAddView(CreateView):
    template_name = 'chrono/manager_agenda_form.html'
    model = Agenda
    fields = ['label']

agenda_add = AgendaAddView.as_view()


class AgendaEditView(UpdateView):
    template_name = 'chrono/manager_agenda_form.html'
    model = Agenda
    fields = ['label']

agenda_edit = AgendaEditView.as_view()


class AgendaDeleteView(DeleteView):
    template_name = 'chrono/manager_confirm_delete.html'
    model = Agenda
    success_url = reverse_lazy('chrono-manager-homepage')

agenda_delete = AgendaDeleteView.as_view()


class AgendaView(DetailView):
    template_name = 'chrono/manager_agenda_view.html'
    model = Agenda

agenda_view = AgendaView.as_view()


class AgendaAddEventView(CreateView):
    template_name = 'chrono/manager_event_form.html'
    model = Event
    form_class = EventForm

    def get_success_url(self):
        return reverse('chrono-manager-agenda-view', kwargs={'pk': self.kwargs.get('pk')})

    def get_initial(self):
        initial = super(AgendaAddEventView, self).get_initial()
        initial['agenda'] = self.kwargs.get('pk')
        return initial

    def get_context_data(self, **kwargs):
        context = super(AgendaAddEventView, self).get_context_data(**kwargs)
        context['agenda'] = Agenda.objects.get(id=self.kwargs.get('pk'))
        return context

agenda_add_event = AgendaAddEventView.as_view()


class EventEditView(UpdateView):
    template_name = 'chrono/manager_event_form.html'
    model = Event
    form_class = EventForm

    def get_context_data(self, **kwargs):
        context = super(EventEditView, self).get_context_data(**kwargs)
        context['agenda'] = self.object.agenda
        return context

    def get_success_url(self):
        return reverse('chrono-manager-agenda-view', kwargs={'pk': self.object.agenda_id})


event_edit = EventEditView.as_view()

def menu_json(request):
    response = HttpResponse(content_type='application/json')
    label = _('Agendas')
    json_str = json.dumps([{'label': force_text(label),
        'slug': 'calendar',
        'url': request.build_absolute_uri(reverse('chrono-manager-homepage'))
        }])
    for variable in ('jsonpCallback', 'callback'):
        if variable in request.GET:
            json_str = '%s(%s);' % (request.GET[variable], json_str)
            break
    response.write(json_str)
    return response
